@@ -1,21 +1,21 @@
-# coding: utf-8
-'''
+"""
 A collection of mixins useful for the various *Client interfaces
-'''
+"""
+
 
-# Import Python libs
-from __future__ import absolute_import, print_function, with_statement, unicode_literals
-import fnmatch
-import signal
-import logging
-import weakref
-import traceback
-import collections
 import copy as pycopy
+import fnmatch
+import logging
+import signal
+import traceback
+import weakref
+from collections.abc import Mapping, MutableMapping
 
-# Import Salt libs
 import salt.exceptions
+import salt.ext.tornado.stack_context
+import salt.log.setup
 import salt.minion
+import salt.transport.client
 import salt.utils.args
 import salt.utils.doc
 import salt.utils.error
@@ -28,44 +28,42 @@ import salt.utils.process
 import salt.utils.state
 import salt.utils.user
 import salt.utils.versions
-import salt.transport.client
-import salt.log.setup
 from salt.ext import six
 
-# Import 3rd-party libs
-import salt.ext.tornado.stack_context
-
 log = logging.getLogger(__name__)
 
-CLIENT_INTERNAL_KEYWORDS = frozenset([
-    'client',
-    'cmd',
-    'eauth',
-    'fun',
-    'kwarg',
-    'match',
-    'token',
-    '__jid__',
-    '__tag__',
-    '__user__',
-    'username',
-    'password',
-    'full_return',
-    'print_event'
-])
+CLIENT_INTERNAL_KEYWORDS = frozenset(
+    [
+        "client",
+        "cmd",
+        "eauth",
+        "fun",
+        "kwarg",
+        "match",
+        "token",
+        "__jid__",
+        "__tag__",
+        "__user__",
+        "username",
+        "password",
+        "full_return",
+        "print_event",
+    ]
+)
 
 
-class ClientFuncsDict(collections.MutableMapping):
-    '''
+class ClientFuncsDict(MutableMapping):
+    """
     Class to make a read-only dict for accessing runner funcs "directly"
-    '''
+    """
+
     def __init__(self, client):
         self.client = client
 
     def __getattr__(self, attr):
-        '''
+        """
         Provide access eg. to 'pack'
-        '''
+        """
         return getattr(self.client.functions, attr)
 
     def __setitem__(self, key, val):
@@ -75,38 +73,40 @@ class ClientFuncsDict(collections.MutableMapping):
         raise NotImplementedError()
 
     def __getitem__(self, key):
-        '''
+        """
         Return a function that you can call with regular func params, but
         will do all the _proc_function magic
-        '''
+        """
         if key not in self.client.functions:
             raise KeyError
 
         def wrapper(*args, **kwargs):
-            low = {'fun': key,
-                   'args': args,
-                   'kwargs': kwargs,
-                   }
+            low = {
+                "fun": key,
+                "args": args,
+                "kwargs": kwargs,
+            }
             pub_data = {}
             # Copy kwargs keys so we can iterate over and pop the pub data
             kwargs_keys = list(kwargs)
 
             # pull out pub_data if you have it
             for kwargs_key in kwargs_keys:
-                if kwargs_key.startswith('__pub_'):
+                if kwargs_key.startswith("__pub_"):
                     pub_data[kwargs_key] = kwargs.pop(kwargs_key)
 
-            async_pub = self.client._gen_async_pub(pub_data.get('__pub_jid'))
+            async_pub = self.client._gen_async_pub(pub_data.get("__pub_jid"))
 
             user = salt.utils.user.get_specific_user()
             return self.client._proc_function(
                 key,
                 low,
                 user,
-                async_pub['tag'],  # TODO: fix
-                async_pub['jid'],  # TODO: fix
+                async_pub["tag"],  # TODO: fix
+                async_pub["jid"],  # TODO: fix
                 False,  # Don't daemonize
             )
+
         return wrapper
 
     def __len__(self):
@@ -116,38 +116,39 @@ class ClientFuncsDict(collections.MutableMapping):
         return iter(self.client.functions)
 
 
-class SyncClientMixin(object):
-    '''
+class SyncClientMixin:
+    """
     A mixin for *Client interfaces to abstract common function execution
-    '''
+    """
+
     functions = ()
 
     def functions_dict(self):
-        '''
+        """
         Return a dict that will mimic the "functions" dict used all over salt.
         It creates a wrapper around the function allowing **kwargs, and if pub_data
         is passed in as kwargs, will re-use the JID passed in
-        '''
+        """
         return ClientFuncsDict(self)
 
     def master_call(self, **kwargs):
-        '''
+        """
         Execute a function through the master network interface.
-        '''
+        """
         load = kwargs
-        load['cmd'] = self.client
+        load["cmd"] = self.client
 
-        with salt.transport.client.ReqChannel.factory(self.opts,
-                                                      crypt='clear',
-                                                      usage='master_call') as channel:
+        with salt.transport.client.ReqChannel.factory(
+            self.opts, crypt="clear", usage="master_call"
+        ) as channel:
             ret = channel.send(load)
-            if isinstance(ret, collections.Mapping):
-                if 'error' in ret:
-                    salt.utils.error.raise_error(**ret['error'])
+            if isinstance(ret, Mapping):
+                if "error" in ret:
+                    salt.utils.error.raise_error(**ret["error"])
             return ret
 
     def cmd_sync(self, low, timeout=None, full_return=False):
-        '''
+        """
         Execute a runner function synchronously; eauth is respected
 
         This function requires that :conf_master:`external_auth` is configured
@@ -161,23 +162,36 @@ class SyncClientMixin(object):
                 'password': 'saltdev',
                 'eauth': 'pam',
             })
-        '''
-        with salt.utils.event.get_master_event(self.opts, self.opts['sock_dir'], listen=True) as event:
+        """
+        with salt.utils.event.get_master_event(
+            self.opts, self.opts["sock_dir"], listen=True
+        ) as event:
             job = self.master_call(**low)
-            ret_tag = salt.utils.event.tagify('ret', base=job['tag'])
+            ret_tag = salt.utils.event.tagify("ret", base=job["tag"])
 
             if timeout is None:
-                timeout = self.opts.get('rest_timeout', 300)
-            ret = event.get_event(tag=ret_tag, full=True, wait=timeout, auto_reconnect=True)
+                timeout = self.opts.get("rest_timeout", 300)
+            ret = event.get_event(
+                tag=ret_tag, full=True, wait=timeout, auto_reconnect=True
+            )
             if ret is None:
                 raise salt.exceptions.SaltClientTimeout(
-                    "RunnerClient job '{0}' timed out".format(job['jid']),
-                    jid=job['jid'])
+                    "RunnerClient job '{}' timed out".format(job["jid"]),
+                    jid=job["jid"],
+                )
 
-            return ret if full_return else ret['data']['return']
+            return ret if full_return else ret["data"]["return"]
 
-    def cmd(self, fun, arg=None, pub_data=None, kwarg=None, print_event=True, full_return=False):
-        '''
+    def cmd(
+        self,
+        fun,
+        arg=None,
+        pub_data=None,
+        kwarg=None,
+        print_event=True,
+        full_return=False,
+    ):
+        """
         Execute a function
 
         .. code-block:: python
@@ -204,72 +218,71 @@ class SyncClientMixin(object):
                 },
             }
 
-        '''
+        """
         if arg is None:
             arg = tuple()
         if not isinstance(arg, list) and not isinstance(arg, tuple):
             raise salt.exceptions.SaltInvocationError(
-                'arg must be formatted as a list/tuple'
+                "arg must be formatted as a list/tuple"
             )
         if pub_data is None:
             pub_data = {}
         if not isinstance(pub_data, dict):
             raise salt.exceptions.SaltInvocationError(
-                'pub_data must be formatted as a dictionary'
+                "pub_data must be formatted as a dictionary"
             )
         if kwarg is None:
             kwarg = {}
         if not isinstance(kwarg, dict):
             raise salt.exceptions.SaltInvocationError(
-                'kwarg must be formatted as a dictionary'
+                "kwarg must be formatted as a dictionary"
             )
         arglist = salt.utils.args.parse_input(
-            arg,
-            no_parse=self.opts.get('no_parse', []))
+            arg, no_parse=self.opts.get("no_parse", [])
+        )
 
         # if you were passed kwarg, add it to arglist
         if kwarg:
-            kwarg['__kwarg__'] = True
+            kwarg["__kwarg__"] = True
             arglist.append(kwarg)
 
         args, kwargs = salt.minion.load_args_and_kwargs(
             self.functions[fun], arglist, pub_data
         )
-        low = {'fun': fun,
-               'arg': args,
-               'kwarg': kwargs}
+        low = {"fun": fun, "arg": args, "kwarg": kwargs}
         return self.low(fun, low, print_event=print_event, full_return=full_return)
 
     @property
     def mminion(self):
-        if not hasattr(self, '_mminion'):
-            self._mminion = salt.minion.MasterMinion(self.opts, states=False, rend=False)
+        if not hasattr(self, "_mminion"):
+            self._mminion = salt.minion.MasterMinion(
+                self.opts, states=False, rend=False
+            )
         return self._mminion
 
     @property
     def store_job(self):
-        '''
+        """
         Helper that allows us to turn off storing jobs for different classes
         that may incorporate this mixin.
-        '''
+        """
         try:
             class_name = self.__class__.__name__.lower()
         except AttributeError:
             log.warning(
-                'Unable to determine class name',
-                exc_info_on_loglevel=logging.DEBUG
+                "Unable to determine class name", exc_info_on_loglevel=logging.DEBUG
             )
             return True
 
         try:
-            return self.opts['{0}_returns'.format(class_name)]
+            return self.opts["{}_returns".format(class_name)]
         except KeyError:
             # No such option, assume this isn't one we care about gating and
             # just return True.
             return True
 
     def low(self, fun, low, print_event=True, full_return=False):
-        '''
+        """
         Execute a function from low data
         Low data includes:
             required:
@@ -280,23 +293,24 @@ class SyncClientMixin(object):
                 - __user__: user who is running the command
                 - __jid__: jid to run under
                 - __tag__: tag to run under
-        '''
+        """
         # fire the mminion loading (if not already done) here
         # this is not to clutter the output with the module loading
         # if we have a high debug level.
         self.mminion  # pylint: disable=W0104
-        jid = low.get('__jid__', salt.utils.jid.gen_jid(self.opts))
-        tag = low.get('__tag__', salt.utils.event.tagify(jid, prefix=self.tag_prefix))
+        jid = low.get("__jid__", salt.utils.jid.gen_jid(self.opts))
+        tag = low.get("__tag__", salt.utils.event.tagify(jid, prefix=self.tag_prefix))
 
-        data = {'fun': '{0}.{1}'.format(self.client, fun),
-                'jid': jid,
-                'user': low.get('__user__', 'UNKNOWN'),
-               }
+        data = {
+            "fun": "{}.{}".format(self.client, fun),
+            "jid": jid,
+            "user": low.get("__user__", "UNKNOWN"),
+        }
 
         if print_event:
-            print_func = self.print_async_event \
-                if hasattr(self, 'print_async_event') \
-                else None
+            print_func = (
+                self.print_async_event if hasattr(self, "print_async_event") else None
+            )
         else:
             # Suppress printing of return event (this keeps us from printing
             # runner/wheel output during orchestration).
@@ -304,25 +318,26 @@ class SyncClientMixin(object):
 
         with salt.utils.event.NamespacedEvent(
             salt.utils.event.get_event(
-                'master',
-                self.opts['sock_dir'],
-                self.opts['transport'],
+                "master",
+                self.opts["sock_dir"],
+                self.opts["transport"],
                 opts=self.opts,
                 listen=False,
             ),
             tag,
-            print_func=print_func
+            print_func=print_func,
         ) as namespaced_event:
 
             # TODO: test that they exist
             # TODO: Other things to inject??
-            func_globals = {'__jid__': jid,
-                            '__user__': data['user'],
-                            '__tag__': tag,
-                            # weak ref to avoid the Exception in interpreter
-                            # teardown of event
-                            '__jid_event__': weakref.proxy(namespaced_event),
-                            }
+            func_globals = {
+                "__jid__": jid,
+                "__user__": data["user"],
+                "__tag__": tag,
+                # weak ref to avoid the Exception in interpreter
+                # teardown of event
+                "__jid_event__": weakref.proxy(namespaced_event),
+            }
 
             try:
                 self_functions = pycopy.copy(self.functions)
@@ -332,14 +347,14 @@ class SyncClientMixin(object):
                 # namespace only once per module-- not per func
                 completed_funcs = []
 
-                for mod_name in six.iterkeys(self_functions):
-                    if '.' not in mod_name:
+                for mod_name in self_functions.keys():
+                    if "." not in mod_name:
                         continue
-                    mod, _ = mod_name.split('.', 1)
+                    mod, _ = mod_name.split(".", 1)
                     if mod in completed_funcs:
                         continue
                     completed_funcs.append(mod)
-                    for global_key, value in six.iteritems(func_globals):
+                    for global_key, value in func_globals.items():
                         self.functions[mod_name].__globals__[global_key] = value
 
                 # There are some discrepancies of what a "low" structure is in the
@@ -351,110 +366,124 @@ class SyncClientMixin(object):
                 # we make the transition we will load "kwargs" using format_call if
                 # there are no kwargs in the low object passed in.
 
-                if 'arg' in low and 'kwarg' in low:
-                    args = low['arg']
-                    kwargs = low['kwarg']
+                if "arg" in low and "kwarg" in low:
+                    args = low["arg"]
+                    kwargs = low["kwarg"]
                 else:
                     f_call = salt.utils.args.format_call(
                         self.functions[fun],
                         low,
-                        expected_extra_kws=CLIENT_INTERNAL_KEYWORDS
+                        expected_extra_kws=CLIENT_INTERNAL_KEYWORDS,
                     )
-                    args = f_call.get('args', ())
-                    kwargs = f_call.get('kwargs', {})
+                    args = f_call.get("args", ())
+                    kwargs = f_call.get("kwargs", {})
 
                 # Update the event data with loaded args and kwargs
-                data['fun_args'] = list(args) + ([kwargs] if kwargs else [])
-                func_globals['__jid_event__'].fire_event(data, 'new')
+                data["fun_args"] = list(args) + ([kwargs] if kwargs else [])
+                func_globals["__jid_event__"].fire_event(data, "new")
 
                 # Initialize a context for executing the method.
-                with salt.ext.tornado.stack_context.StackContext(self.functions.context_dict.clone):
+                with salt.ext.tornado.stack_context.StackContext(
+                    self.functions.context_dict.clone
+                ):
                     func = self.functions[fun]
                     try:
-                        data['return'] = func(*args, **kwargs)
+                        data["return"] = func(*args, **kwargs)
                     except TypeError as exc:
-                        data['return'] = '\nPassed invalid arguments: {0}\n\nUsage:\n{1}'.format(exc, func.__doc__)
+                        data[
+                            "return"
+                        ] = "\nPassed invalid arguments: {}\n\nUsage:\n{}".format(
+                            exc, func.__doc__
+                        )
                     try:
-                        data['success'] = self.context.get('retcode', 0) == 0
+                        data["success"] = self.context.get("retcode", 0) == 0
                     except AttributeError:
                         # Assume a True result if no context attribute
-                        data['success'] = True
-                    if isinstance(data['return'], dict) and 'data' in data['return']:
+                        data["success"] = True
+                    if isinstance(data["return"], dict) and "data" in data["return"]:
                         # some functions can return boolean values
-                        data['success'] = salt.utils.state.check_result(data['return']['data'])
+                        data["success"] = salt.utils.state.check_result(
+                            data["return"]["data"]
+                        )
             except (Exception, SystemExit) as ex:  # pylint: disable=broad-except
                 if isinstance(ex, salt.exceptions.NotImplemented):
-                    data['return'] = six.text_type(ex)
+                    data["return"] = str(ex)
                 else:
-                    data['return'] = 'Exception occurred in {0} {1}: {2}'.format(
-                        self.client,
-                        fun,
-                        traceback.format_exc(),
-                        )
-                data['success'] = False
+                    data["return"] = "Exception occurred in {} {}: {}".format(
+                        self.client, fun, traceback.format_exc(),
+                    )
+                data["success"] = False
 
             if self.store_job:
                 try:
                     salt.utils.job.store_job(
                         self.opts,
                         {
-                            'id': self.opts['id'],
-                            'tgt': self.opts['id'],
-                            'jid': data['jid'],
-                            'return': data,
+                            "id": self.opts["id"],
+                            "tgt": self.opts["id"],
+                            "jid": data["jid"],
+                            "return": data,
                         },
                         event=None,
                         mminion=self.mminion,
-                        )
+                    )
                 except salt.exceptions.SaltCacheError:
-                    log.error('Could not store job cache info. '
-                              'Job details for this run may be unavailable.')
+                    log.error(
+                        "Could not store job cache info. "
+                        "Job details for this run may be unavailable."
+                    )
 
             # Outputters _can_ mutate data so write to the job cache first!
-            namespaced_event.fire_event(data, 'ret')
+            namespaced_event.fire_event(data, "ret")
 
             # if we fired an event, make sure to delete the event object.
             # This will ensure that we call destroy, which will do the 0MQ linger
-            log.info('Runner completed: %s', data['jid'])
-            return data if full_return else data['return']
+            log.info("Runner completed: %s", data["jid"])
+            return data if full_return else data["return"]
 
     def get_docs(self, arg=None):
-        '''
+        """
         Return a dictionary of functions and the inline documentation for each
-        '''
+        """
         if arg:
-            if '*' in arg:
+            if "*" in arg:
                 target_mod = arg
                 _use_fnmatch = True
             else:
-                target_mod = arg + '.' if not arg.endswith('.') else arg
+                target_mod = arg + "." if not arg.endswith(".") else arg
                 _use_fnmatch = False
             if _use_fnmatch:
-                docs = [(fun, self.functions[fun].__doc__)
-                        for fun in fnmatch.filter(self.functions, target_mod)]
+                docs = [
+                    (fun, self.functions[fun].__doc__)
+                    for fun in fnmatch.filter(self.functions, target_mod)
+                ]
             else:
-                docs = [(fun, self.functions[fun].__doc__)
-                        for fun in sorted(self.functions)
-                        if fun == arg or fun.startswith(target_mod)]
+                docs = [
+                    (fun, self.functions[fun].__doc__)
+                    for fun in sorted(self.functions)
+                    if fun == arg or fun.startswith(target_mod)
+                ]
         else:
-            docs = [(fun, self.functions[fun].__doc__)
-                    for fun in sorted(self.functions)]
+            docs = [
+                (fun, self.functions[fun].__doc__) for fun in sorted(self.functions)
+            ]
         docs = dict(docs)
         return salt.utils.doc.strip_rst(docs)
 
 
-class AsyncClientMixin(object):
-    '''
+class AsyncClientMixin:
+    """
     A mixin for *Client interfaces to enable easy asynchronous function execution
-    '''
+    """
+
     client = None
     tag_prefix = None
 
     def _proc_function_remote(self, fun, low, user, tag, jid, daemonize=True):
-        '''
+        """
         Run this method in a multiprocess target to execute the function on the
         master and fire the return data on the event bus
-        '''
+        """
         if daemonize and not salt.utils.platform.is_windows():
             # Shutdown the multiprocessing before daemonizing
             salt.log.setup.shutdown_multiprocessing_logging()
@@ -465,9 +494,9 @@ class AsyncClientMixin(object):
             salt.log.setup.setup_multiprocessing_logging()
 
         # pack a few things into low
-        low['__jid__'] = jid
-        low['__user__'] = user
-        low['__tag__'] = tag
+        low["__jid__"] = jid
+        low["__user__"] = user
+        low["__tag__"] = tag
 
         try:
             return self.cmd_sync(low)
@@ -475,10 +504,31 @@ class AsyncClientMixin(object):
             log.error(exc)
 
     def _proc_function(self, fun, low, user, tag, jid, daemonize=True):
-        '''
+        """
         Run this method in a multiprocess target to execute the function
         locally and fire the return data on the event bus
-        '''
+        """
+        if daemonize and not salt.utils.platform.is_windows():
+            # Shutdown the multiprocessing before daemonizing
+            salt.log.setup.shutdown_multiprocessing_logging()
+
+            salt.utils.process.daemonize()
+
+            # Reconfigure multiprocessing logging after daemonizing
+            salt.log.setup.setup_multiprocessing_logging()
+
+        # pack a few things into low
+        low["__jid__"] = jid
+        low["__user__"] = user
+        low["__tag__"] = tag
+
+        return self.low(fun, low)
+
+    def _proc_function_local(self, fun, low, user, tag, jid, daemonize=True):
+        """
+        Run this method in a multiprocess target to execute the function
+        locally and fire the return data on the event bus
+        """
         if daemonize and not salt.utils.platform.is_windows():
             # Shutdown the multiprocessing before daemonizing
             salt.log.setup.shutdown_multiprocessing_logging()
@@ -496,7 +546,7 @@ class AsyncClientMixin(object):
         return self.low(fun, low)
 
     def cmd_async(self, low):
-        '''
+        """
         Execute a function asynchronously; eauth is respected
 
         This function requires that :conf_master:`external_auth` is configured
@@ -512,20 +562,20 @@ class AsyncClientMixin(object):
                 'password': 'saltdev',
             })
             {'jid': '20131219224744416681', 'tag': 'salt/wheel/20131219224744416681'}
-        '''
+        """
         return self.master_call(**low)
 
     def _gen_async_pub(self, jid=None):
         if jid is None:
             jid = salt.utils.jid.gen_jid(self.opts)
         tag = salt.utils.event.tagify(jid, prefix=self.tag_prefix)
-        return {'tag': tag, 'jid': jid}
+        return {"tag": tag, "jid": jid}
 
     def asynchronous(self, fun, low, user="UNKNOWN", pub=None, local=True):
-        '''
+        """
         Execute the function in a multiprocess and return the event tag to use
         to watch for the return
-        '''
+        """
         if local:
             proc_func = self._proc_function
         else:
@@ -544,36 +594,41 @@ class AsyncClientMixin(object):
         return async_pub
 
     def print_async_event(self, suffix, event):
-        '''
+        """
         Print all of the events with the prefix 'tag'
-        '''
+        """
         if not isinstance(event, dict):
             return
 
         # if we are "quiet", don't print
-        if self.opts.get('quiet', False):
+        if self.opts.get("quiet", False):
             return
 
         # some suffixes we don't want to print
-        if suffix in ('new',):
+        if suffix in ("new",):
             return
 
         try:
-            outputter = self.opts.get('output', event.get('outputter', None) or event.get('return').get('outputter'))
+            outputter = self.opts.get(
+                "output",
+                event.get("outputter", None) or event.get("return").get("outputter"),
+            )
         except AttributeError:
             outputter = None
 
         # if this is a ret, we have our own set of rules
-        if suffix == 'ret':
+        if suffix == "ret":
             # Check if outputter was passed in the return data. If this is the case,
             # then the return data will be a dict two keys: 'data' and 'outputter'
-            if isinstance(event.get('return'), dict) \
-                    and set(event['return']) == set(('data', 'outputter')):
-                event_data = event['return']['data']
-                outputter = event['return']['outputter']
+            if isinstance(event.get("return"), dict) and set(event["return"]) == {
+                "data",
+                "outputter",
+            }:
+                event_data = event["return"]["data"]
+                outputter = event["return"]["outputter"]
             else:
-                event_data = event['return']
+                event_data = event["return"]
         else:
-            event_data = {'suffix': suffix, 'event': event}
+            event_data = {"suffix": suffix, "event": event}
 
         salt.output.display_output(event_data, outputter, self.opts)
